module FiForm
  class << self
    attr_accessor :configuration
  end

  def self.configure
    self.configuration ||= Configuration.new
    yield(configuration)
  end

  class Configuration
    attr_accessor :renderers, :default

    def initialize
      @renderers = {}
    end

    # Register a renderer. The last added renderer will be the default if not specified
    def register(options)
      self.renderers.merge! options
    end

    # add custom behaviour
    def add_builder_method(name, &block)
      FiForm::Builder.add_builder_method name, &block
    end

    # fetch a renderer. if none is specified, the default or debug renderer will be returned
    def get_renderer(identifier=nil)
      identifier ||= default_renderer
      self.renderers[identifier] || DebugRenderer
    end

    # reload renderer classes in development
    def reload!
      self.renderers = self.renderers.map{|k, klass| [k, klass.to_s.constantize] }.to_h # uncovered
    end

  protected
    def default_renderer
      @default || self.renderers.keys.last
    end
  end

end