class FiForm::Item < Struct.new(:kind, :options)

  class Input < FiForm::Item
    def initialize(*args)
      super
      if form.errors.key?(self.field)
        self.options[:errors] = self.form.errors[self.field]
      end
    end
  end

  class Form < FiForm::Item
    def as_json(*args)
      super(except: :resource)
    end
  end

  def self.build(kind, options)
    case kind
    when :input
      Input
    when :form
      Form
    else
      self
    end.new(kind, options)
  end

  attr_reader :resource, :form, :field
  alias_method :object, :resource
  def initialize(*args)
    super
    @form = self.options.delete(:form)

    @resource = if self.options.key? :resource
      self.options.delete(:resource)
    else
      @form.resource
    end
  end

  def errors
    self.options.fetch(:errors)
  end

  def items
    self.options&.dig(:items) || []
  end
  alias_method :children, :items

  def field
    self.options.fetch(:field)
  end

  def value
    self.options.fetch(:value)
  end

  def input
    self.options&.dig(:input)
  end

  def <<(item)
    self.options[:items] ||= []
    self.options[:items] << item
  end

  def add_items(items)
    self.options[:items] += items
  end

  def last_child
    self.options[:items].last
  end

  def as_json(*args)
    super(except: :resource)
  end

end